@@ -2,7 +2,7 @@ package Importer;
 use strict qw/vars subs/; # Not refs!
 use warnings; no warnings 'once';
 
-our $VERSION = '0.013';
+our $VERSION = '0.025';
 
 my %SIG_TO_SLOT = (
     '&' => 'CODE',
@@ -17,7 +17,31 @@ our %IMPORTED;
 # This will be used to check if an import arg is a version number
 my %NUMERIC = map +($_ => 1), 0 .. 9;
 
-sub IMPORTER_MENU() {(export_ok => [qw/optimal_import/])}
+sub IMPORTER_MENU() {
+    return (
+        export_ok   => [qw/optimal_import/],
+        export_anon => {
+            import => sub {
+                my $from  = shift;
+                my @caller = caller(0);
+
+                _version_check($from, \@caller, shift @_) if @_ && $NUMERIC{substr($_[0], 0, 1)};
+
+                my $file = _mod_to_file($from);
+                _load_file(\@caller, $file) unless $INC{$file};
+
+                return if optimal_import($from, $caller[0], \@caller, @_);
+
+                my $self = __PACKAGE__->new(
+                    from   => $from,
+                    caller => \@caller,
+                );
+
+                $self->do_import($caller[0], @_);
+            },
+        },
+    );
+}
 
 ###########################################################################
 #
@@ -72,7 +96,11 @@ sub import_into {
 
     my @caller;
 
-    if ($into =~ m/^\d+$/) {
+    if (ref($into)) {
+        @caller = @$into;
+        $into = $caller[0];
+    }
+    elsif ($into =~ m/^\d+$/) {
         @caller = caller($into + 1);
         $into = $caller[0];
     }
@@ -302,50 +330,60 @@ sub reload_menu {
 
     my $from = $self->from;
 
-    my (
-        $export,
-        $export_ok,
-        $export_tags,
-        $export_fail,
-        $generate,
-        $export_gen,
-        $export_anon,
-        $export_magic,
-        $new_style
-    );
-
     if (my $menu_sub = *{"$from\::IMPORTER_MENU"}{CODE}) {
         # Hook, other exporter modules can define this method to be compatible with
         # Importer.pm
 
-        $new_style = 1;
-
         my %got = $from->$menu_sub($into, $self->get_caller);
-        $export       = $got{export}       || [];
-        $export_ok    = $got{export_ok}    || [];
-        $export_tags  = $got{export_tags}  || {};
-        $export_fail  = $got{export_fail}  || [];
-        $export_anon  = $got{export_anon}  || {};
-        $export_magic = $got{export_magic} || {};
 
-        $export_gen = $got{export_gen};
-        $generate   = $got{generate};
+        $got{export}       ||= [];
+        $got{export_ok}    ||= [];
+        $got{export_tags}  ||= {};
+        $got{export_fail}  ||= [];
+        $got{export_anon}  ||= {};
+        $got{export_magic} ||= {};
 
         $self->croak("'$from' provides both 'generate' and 'export_gen' in its IMPORTER_MENU (They are exclusive, module must pick 1)")
-            if $export_gen && $generate;
+            if $got{export_gen} && $got{generate};
 
-        $export_gen ||= {};
+        $got{export_gen} ||= {};
+
+        $self->{menu} = $self->_build_menu($into => \%got, 1);
     }
     else {
-        $export       = \@{"$from\::EXPORT"};
-        $export_ok    = \@{"$from\::EXPORT_OK"};
-        $export_tags  = \%{"$from\::EXPORT_TAGS"};
-        $export_fail  = \@{"$from\::EXPORT_FAIL"};
-        $export_gen   = \%{"$from\::EXPORT_GEN"};
-        $export_anon  = \%{"$from\::EXPORT_ANON"};
-        $export_magic = \%{"$from\::EXPORT_MAGIC"};
+        my %got;
+        $got{export}        = \@{"$from\::EXPORT"};
+        $got{export_ok}     = \@{"$from\::EXPORT_OK"};
+        $got{export_tags}   = \%{"$from\::EXPORT_TAGS"};
+        $got{export_fail}   = \@{"$from\::EXPORT_FAIL"};
+        $got{export_gen}    = \%{"$from\::EXPORT_GEN"};
+        $got{export_anon}   = \%{"$from\::EXPORT_ANON"};
+        $got{export_magic}  = \%{"$from\::EXPORT_MAGIC"};
+
+        $self->{menu} = $self->_build_menu($into => \%got, 0);
     }
 
+    $self->{menu_for} = $into;
+
+    return $self->{menu};
+}
+
+sub _build_menu {
+    my $self = shift;
+    my ($into, $got, $new_style) = @_;
+
+    my $from = $self->from;
+
+    my $export       = $got->{export}       || [];
+    my $export_ok    = $got->{export_ok}    || [];
+    my $export_tags  = $got->{export_tags}  || {};
+    my $export_fail  = $got->{export_fail}  || [];
+    my $export_anon  = $got->{export_anon}  || {};
+    my $export_gen   = $got->{export_gen}   || {};
+    my $export_magic = $got->{export_magic} || {};
+
+    my $generate = $got->{generate};
+
     $generate ||= sub {
         my $symbol = shift;
         my ($sig, $name) = ($symbol =~ m/^(\W?)(.*)$/);
@@ -359,7 +397,7 @@ sub reload_menu {
         $from->$do($into, $symbol);
     } if $export_gen && keys %$export_gen;
 
-    my $lookup = {};
+    my $lookup  = {};
     my $exports = {};
     for my $sym (@$export, @$export_ok, keys %$export_gen, keys %$export_anon) {
         my ($sig, $name) = ($sym =~ m/^(\W?)(.*)$/);
@@ -370,6 +408,7 @@ sub reload_menu {
 
         next if $export_gen->{"${sig}${name}"};
         next if $sig eq '&' && $export_gen->{$name};
+        next if $got->{generate} && $generate->("${sig}${name}");
 
         my $fqn = "$from\::$name";
         # We cannot use *{$fqn}{TYPE} here, it breaks for autoloaded subs, this
@@ -400,6 +439,9 @@ sub reload_menu {
         'DEFAULT' => [ @$export ],
     };
 
+    # Add 'ALL' tag unless already specified. We want to normalize it.
+    $tags->{ALL} ||= [ sort grep {m/^[\&\$\@\%\*]/} keys %$lookup ];
+
     my $fail = @$export_fail ? {
         map {
             my ($sig, $name) = (m/^(\W?)(.*)$/);
@@ -408,8 +450,7 @@ sub reload_menu {
         } @$export_fail
     } : undef;
 
-    $self->{menu_for} = $into;
-    return $self->{menu} = {
+    my $menu = {
         lookup   => $lookup,
         exports  => $exports,
         tags     => $tags,
@@ -417,45 +458,63 @@ sub reload_menu {
         generate => $generate,
         magic    => $export_magic,
     };
+
+    return $menu;
 }
 
 sub parse_args {
     my $self = shift;
     my ($into, @args) = @_;
 
-    @args = (':DEFAULT') unless @args;
+    my $menu = $self->menu($into);
+
+    my @out = $self->_parse_args($into, $menu, \@args);
+    pop @out;
+    return @out;
+}
+
+sub _parse_args {
+    my $self = shift;
+    my ($into, $menu, $args, $is_tag) = @_;
 
     my $from = $self->from;
-    my $menu = $self->menu($into);
+    my $main_menu = $self->menu($into);
+    $menu ||= $main_menu;
+
+    # First we strip out versions numbers and setters, this simplifies the logic late.
+    my @sets;
+    my @versions;
+    my @leftover;
+    for my $arg (@$args) {
+        no warnings 'void';
+
+        # Code refs are custom setters
+        # If the first character is an ASCII numeric then it is a version number
+        push @sets     => $arg and next if ref($arg) eq 'CODE';
+        push @versions => $arg xor next if $NUMERIC{substr($arg, 0, 1)};
+        push @leftover => $arg;
+    }
+
+    $self->carp("Multiple setters specified, only 1 will be used") if @sets > 1;
+    my $set = pop @sets;
+
+    $args = \@leftover;
+    @$args = (':DEFAULT') unless $is_tag || @$args || @versions;
 
     my %exclude;
     my @import;
-    my @versions;
-    my $set;
 
-    while(my $full_arg = shift @args) {
+    while(my $full_arg = shift @$args) {
         my $arg = $full_arg;
         my $lead = substr($arg, 0, 1);
+
         my ($spec, $exc);
-
-        # If the first character is an ASCII numeric then it is a version number
-        if ($NUMERIC{$lead}) {
-            push @versions => $arg;
-            next;
-        }
-        elsif(ref($arg) eq 'CODE') {
-            $self->carp("Multiple setters specified, only 1 will be used") if $set;
-            $set = $arg;
-            unshift @args => ':DEFAULT' unless @args || @import || keys %exclude || @versions;
-            next;
-        }
-
         if ($lead eq '!') {
             $exc = $lead;
 
             if ($arg eq '!') {
                 # If the current arg is just '!' then we are negating the next item.
-                $arg = shift @args;
+                $arg = shift @$args;
             }
             else {
                 # Strip off the '!'
@@ -464,7 +523,7 @@ sub parse_args {
 
             # Exporter.pm legacy behavior
             # negated first item implies starting with default set:
-            unshift @args => ':DEFAULT' unless @import || keys %exclude || @versions;
+            unshift @$args => ':DEFAULT' unless @import || keys %exclude || @versions;
 
             # Now we have a new lead character
             $lead = substr($arg, 0, 1);
@@ -472,7 +531,51 @@ sub parse_args {
         else {
             # If the item is followed by a reference then they are asking us to
             # do something special...
-            $spec = ref($args[0]) eq 'HASH' ? shift @args : {};
+            $spec = ref($args->[0]) eq 'HASH' ? shift @$args : {};
+        }
+
+        if($lead eq ':') {
+            substr($arg, 0, 1, '');
+            my $tag = $menu->{tags}->{$arg} or $self->croak("$from does not export the :$arg tag");
+
+            my (undef, $cvers, $cexc, $cimp, $cset, $newmenu) = $self->_parse_args($into, $menu, $tag, $arg);
+
+            $self->croak("Exporter specified version numbers (" . join(', ', @$cvers) . ") in the :$arg tag!")
+                if @$cvers;
+
+            $self->croak("Exporter specified a custom symbol setter in the :$arg tag!")
+                if $cset;
+
+            # Merge excludes
+            %exclude = (%exclude, %$cexc);
+
+            if ($exc) {
+                $exclude{$_} = 1 for grep {!ref($_) && substr($_, 0, 1) ne '+'} map {$_->[0]} @$cimp;
+            }
+            elsif ($spec && keys %$spec) {
+                $self->croak("Cannot use '-as' to rename multiple symbols included by: $full_arg")
+                    if $spec->{'-as'} && @$cimp > 1;
+
+                for my $set (@$cimp) {
+                    my ($sym, $cspec) = @$set;
+
+                    # Start with a blind squash, spec from tag overrides the ones inside.
+                    my $nspec = {%$cspec, %$spec};
+
+                    $nspec->{'-prefix'}  = "$spec->{'-prefix'}$cspec->{'-prefix'}"   if $spec->{'-prefix'}  && $cspec->{'-prefix'};
+                    $nspec->{'-postfix'} = "$cspec->{'-postfix'}$spec->{'-postfix'}" if $spec->{'-postfix'} && $cspec->{'-postfix'};
+
+                    push @import => [$sym, $nspec];
+                }
+            }
+            else {
+                push @import => @$cimp;
+            }
+
+            # New menu
+            $menu = $newmenu;
+
+            next;
         }
 
         # Process the item to figure out what symbols are being touched, if it
@@ -481,11 +584,6 @@ sub parse_args {
         if(ref($arg) eq 'Regexp') {
             @list = sort grep /$arg/, keys %{$menu->{lookup}};
         }
-        elsif($lead eq ':') {
-            substr($arg, 0, 1, '');
-            my $tag = $menu->{tags}->{$arg} or $self->croak("$from does not export the :$arg tag");
-            @list = @$tag;
-        }
         elsif($lead eq '/' && $arg =~ m{^/(.*)/$}) {
             my $pattern = $1;
             @list = sort grep /$1/, keys %{$menu->{lookup}};
@@ -495,9 +593,7 @@ sub parse_args {
         }
 
         # Normalize list, always have a sigil
-        my %seen;
-        @list = grep !$seen{$_}++, map {m/^\W/ ? $_ : "\&$_" } @list;
-
+        @list = map {m/^\W/ ? $_ : "\&$_" } @list;
 
         if ($exc) {
             $exclude{$_} = 1 for @list;
@@ -510,7 +606,7 @@ sub parse_args {
         }
     }
 
-    return ($into, \@versions, \%exclude, \@import, $set);
+    return ($into, \@versions, \%exclude, \@import, $set, $menu);
 }
 
 sub _handle_fail {
@@ -630,11 +726,11 @@ require \$file;
 
 
 my %HEAVY_VARS = (
-    IMPORTER_MENU => 'CODE',  # Origin package has a custom menu
-    EXPORT_FAIL   => 'ARRAY', # Origin package has a failure handler
-    EXPORT_GEN    => 'HASH',  # Origin package has generators
-    EXPORT_ANON   => 'HASH',  # Origin package has anonymous exports
-    EXPORT_MAGIC  => 'HASH',  # Origin package has magic to apply post-export
+    IMPORTER_MENU => 'CODE',     # Origin package has a custom menu
+    EXPORT_FAIL   => 'ARRAY',    # Origin package has a failure handler
+    EXPORT_GEN    => 'HASH',     # Origin package has generators
+    EXPORT_ANON   => 'HASH',     # Origin package has anonymous exports
+    EXPORT_MAGIC  => 'HASH',     # Origin package has magic to apply post-export
 );
 
 sub optimal_import {
@@ -693,7 +789,7 @@ L<Exporter>s specification. The exporter modules themselves do not need to use
 or inherit from the L<Exporter> module, they just need to set C<@EXPORT> and/or
 other variables.
 
-=head1 SYNOPSYS
+=head1 SYNOPSIS
 
     # Import defaults
     use Importer 'Some::Module';
@@ -735,7 +831,7 @@ addition to others.
 
 Adding a feature to L<Exporter> means that any consumer module that relies on
 the new features must depend on a specific version of L<Exporter>. This seems
-somewhat backwords since L<Exporter> is used by the module you are importing
+somewhat backwards since L<Exporter> is used by the module you are importing
 from.
 
 =item Exporter.pm is really old/crazy code
@@ -764,9 +860,9 @@ override this using the C<IMPORTER_MENU()> hook.
 
 =back
 
-=head1 COMPATABILITY
+=head1 COMPATIBILITY
 
-This module aims for 100% compatabilty with every feature of L<Exporter>, plus
+This module aims for 100% compatibility with every feature of L<Exporter>, plus
 added features such as import renaming.
 
 If you find something that works differently, or not at all when compared to
@@ -848,7 +944,7 @@ The original symbol name (with sigil) from the original package.
 =item @SYMBOLS (optional)
 
 Symbols you wish to import. If no symbols are specified then the defaults will
-be used.
+be used. You may also specify tags using the ':' prefix.
 
 =back
 
@@ -860,6 +956,10 @@ You can define/import subsets of symbols using predefined tags.
 
     use Importer 'Some::Thing' => ':tag';
 
+L<Importer> will automatically populate the C<:DEFAULT> tag for you.
+L<Importer> will also give you an C<:ALL> tag with ALL exports so long as the
+exporter does not define a C<:ALL> tag already.
+
 =head2 /PATTERN/ or qr/PATTERN/
 
 You can import all symbols that match a pattern. The pattern can be supplied a
@@ -869,7 +969,7 @@ string starting and ending with '/', or you can provide a C<qr/../> reference.
 
     use Importer 'Some::Thing' => qr/oo/;
 
-=head2 EXLUDING SYMBOLS
+=head2 EXCLUDING SYMBOLS
 
 You can exclude symbols by prefixing them with '!'.
 
@@ -902,7 +1002,7 @@ all symbols from the tag/patterm.
 
 =head2 CUSTOM EXPORT ASSIGNMENT
 
-This lets you provide an alternative to the C<*name = $ref> export assingment.
+This lets you provide an alternative to the C<*name = $ref> export assignment.
 See the list of L<parameters|/"IMPORT PARAMETERS"> to C<import()>
 
 =head2 UNIMPORTING
@@ -996,7 +1096,7 @@ This allows you to export symbols that are generated on export. The key should
 be the name of a symbol. The value should be a coderef that produces a
 reference that will be exported.
 
-When the generators are called they will recieve 2 arguments, the package the
+When the generators are called they will receive 2 arguments, the package the
 symbol is being exported into, and the symbol being imported (name may or may
 not include sigil for subs).
 
@@ -1198,7 +1298,7 @@ This will remove imported symbols from the objects C<from> package. If you
 specify a list of C<@symbols> then only the specified symbols will be removed,
 otherwise all symbols imported using Importer will be removed.
 
-B<Note:> Please be aware fo the difference between C<do_import()> and
+B<Note:> Please be aware of the difference between C<do_import()> and
 C<do_unimport()>. For import 'from' us used as the origin, in unimport it is
 used as the target. This means you cannot re-use an instance to import and then
 unimport.
@@ -1300,7 +1400,7 @@ imports then only the LAST one will be used.
 
 These can be imported:
 
-    use Importer 'Importer' => qw/optimal_import/;
+    use Importer 'Importer' => qw/import optimal_import/;
 
 =over 4
 
@@ -1315,6 +1415,19 @@ If the import is successful this will return true.
 If the import is unsuccessful this will return false, and no modifications to
 the symbol table will occur.
 
+=item $class->import(@imports)
+
+If you write class intended to be used with L<Importer>, but also need to
+provide a legacy C<import()> method for direct consumers of your class, you can
+import this C<import()> method.
+
+    package My::Exporter;
+
+    # This will give you 'import()' much like 'use base "Exporter";'
+    use Importer 'Importer' => qw/import/;
+
+    ...
+
 =back
 
 =head1 SOURCE
